==> spell-0.38.12-py2.py3-none-any.whl: spell/serving/server.py <==
from pathlib import Path
from typing import Callable, IO, Optional, Union

from starlette.applications import Starlette
from starlette.middleware import Middleware
from starlette.middleware.cors import CORSMiddleware
from starlette.middleware.gzip import GZipMiddleware
from starlette.requests import Request
from starlette.responses import JSONResponse, PlainTextResponse, Response
from starlette.routing import Route
import yaml

from spell.serving.exceptions import BadAPIResponse, InvalidServerConfiguration
from spell.serving.api import API
from spell.serving.types import APIResponse

READY_FILE = "/ready"


def make_api_route(func: Callable[[Request], APIResponse]) -> Callable[[Request], APIResponse]:
    async def endpoint(request: Request) -> Response:
        return wrap_response(await func(request))

    return endpoint


def wrap_response(response: APIResponse) -> Response:
    response, tasks = response
    if isinstance(response, bytes):
        return Response(content=response, media_type="application/octet-stream", background=tasks)
    elif isinstance(response, str):
        return PlainTextResponse(content=response, background=tasks)
    elif isinstance(response, Response):
        if tasks is not None and tasks.tasks:
            if response.background:
                response.background.tasks += tasks.tasks
            else:
                response.background = tasks
        return response
    else:
        try:
            return JSONResponse(response, background=tasks)
        except Exception as e:
            raise BadAPIResponse(
                "Invalid response. Return an object that is JSON-serializable (including its "
                "nested fields), a bytes object, a string, or a "
                "starlette.response.Response object"
            ) from e


def make_api(
    config: Union[bytes, IO[bytes], str, IO[str]],
    entrypoint: Optional[str] = None,
    module_path: Optional[str] = None,
    python_path: Optional[str] = None,
    classname: Optional[str] = None,
) -> API:
    if module_path:
        if not python_path:
            raise InvalidServerConfiguration(
                "Python path must be provided if module path is provided"
            )
        module_path = Path(module_path)
        if not module_path.is_dir():
            raise InvalidServerConfiguration("Module path must be a directory")
        api = API.from_module(module_path, python_path, classname=classname)
    elif not entrypoint:
        raise InvalidServerConfiguration("Entrypoint must be provided if module_path is not")
    else:  # TODO(Justin) Remove after entrypoint deletion
        entrypoint = Path(entrypoint)
        if entrypoint.is_file():
            api = API.from_file(entrypoint, classname=classname)
        elif entrypoint.is_dir():
            if not python_path:
                raise InvalidServerConfiguration(
                    "Classpath must be provided if entrypoint is a directory"
                )
            api = API.from_module(entrypoint, python_path, classname=classname)
        else:
            raise InvalidServerConfiguration(
                f"Entrypoint {entrypoint} must be a either a file or directory"
            )
    api.initialize_predictor(yaml.safe_load(config))
    return api


def create_ready_file() -> None:
    open(READY_FILE, "a").close()


def make_app(api: API, debug: bool = False) -> Starlette:
    routes = [
        Route("/predict", make_api_route(api.predict), methods=["POST"]),
        Route("/health", make_api_route(api.health), methods=["GET"]),
    ]

    middleware = [
        Middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_methods=["*"],
            allow_headers=["*"],
            allow_credentials=True,
        ),
        Middleware(GZipMiddleware),
    ]
    return Starlette(
        debug=debug, routes=routes, middleware=middleware, on_startup=[create_ready_file]
    )

==> spell-0.38.15-py2.py3-none-any.whl: spell/serving/server.py <==
from pathlib import Path
from typing import Callable, IO, Optional, Union

from starlette.applications import Starlette
from starlette.middleware import Middleware
from starlette.middleware.cors import CORSMiddleware
from starlette.middleware.gzip import GZipMiddleware
from starlette.requests import Request
from starlette.responses import JSONResponse, PlainTextResponse, Response
from starlette.routing import Route
import yaml

from spell.serving.exceptions import BadAPIResponse, InvalidServerConfiguration
from spell.serving.api import API
from spell.serving.types import APIResponse

READY_FILE = "/ready"


def make_api_route(func: Callable[[Request], APIResponse]) -> Callable[[Request], APIResponse]:
    async def endpoint(request: Request) -> Response:
        return wrap_response(await func(request))

    return endpoint


def wrap_response(response: APIResponse) -> Response:
    response, tasks = response
    if isinstance(response, bytes):
        return Response(content=response, media_type="application/octet-stream", background=tasks)
    elif isinstance(response, str):
        return PlainTextResponse(content=response, background=tasks)
    elif isinstance(response, Response):
        if tasks is not None and tasks.tasks:
            if response.background:
                response.background.tasks += tasks.tasks
            else:
                response.background = tasks
        return response
    else:
        try:
            return JSONResponse(response, background=tasks)
        except Exception as e:
            raise BadAPIResponse(
                "Invalid response. Return an object that is JSON-serializable (including its "
                "nested fields), a bytes object, a string, or a "
                "starlette.response.Response object"
            ) from e


def make_api(
    config: Union[bytes, IO[bytes], str, IO[str]],
    module_path: str,
    python_path: str,
    classname: Optional[str] = None,
) -> API:
    if not module_path:
        raise InvalidServerConfiguration("module_path cannot be an empty string")
    if not python_path:
        raise InvalidServerConfiguration("python_path cannot be an empty string")
    module_path = Path(module_path)
    if not module_path.is_dir():
        raise InvalidServerConfiguration("Module path must be a directory")
    api = API.from_module(module_path, python_path, classname=classname)
    api.initialize_predictor(yaml.safe_load(config))
    return api


def create_ready_file() -> None:
    open(READY_FILE, "a").close()


def make_app(api: API, debug: bool = False) -> Starlette:
    routes = [
        Route("/predict", make_api_route(api.predict), methods=["POST"]),
        Route("/health", make_api_route(api.health), methods=["GET"]),
    ]

    middleware = [
        Middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_methods=["*"],
            allow_headers=["*"],
            allow_credentials=True,
        ),
        Middleware(GZipMiddleware),
    ]
    return Starlette(
        debug=debug, routes=routes, middleware=middleware, on_startup=[create_ready_file]
    )
